#!/usr/bin/env python
#
#       RC4, ARC4, ARCFOUR algorithm
#
#       Copyright (c) 2009 joonis new media
#       Author: Thimo Kraemer <thimo.kraemer@joonis.de>
#       Modified by Roei Schuster for Tel Aviv University course
#       Information Security: Theory vs. Reality.
# 
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#


import sys, random
from math import pow

OFFSET = random.randint(0,pow(2, 31))

def openIndicesFile(fileName):
	global f
	f = open(fileName, 'w')

def closeIndicesFile():
	global f
	f.close()

def readAndLeak(array, indice):
	'''
	Leak indice into indice file, and return array[indice]
	'''
	global f
	f.write((str(indice + OFFSET)) + '\n') # output indice of read access
	return array[indice]

def swap(array, i, j):
	temp = array[i]
	array[i] = array[j]
	array[j] = temp
	
def main():
	if len(sys.argv) != 5:
		print "Usage: rc4outputIndices.py PLAINTEXT_FILE CIPHERTEXT_FILE INDICES_FILE PASSWORD"
		return
	
	#argument parsing
	plaintext = open(sys.argv[1], 'rb').read()
	ciphertext_file_name = sys.argv[2]
	indices_file_name = sys.argv[3]
	password = sys.argv[4]
	
	
	#key ("secret") generation (from password)
	i = 0
	secret = range(256)
	for k in range(256):
		i = (i + secret[k] + ord(password[k % len(password)])) % 256
		secret[k], secret[i] = secret[i], secret[k]
	
	open(r'S.txt','w').writelines(map(lambda x: str(x) + '\n', secret))
	
	#encryption
	openIndicesFile(indices_file_name)
	i = 0
	j = 0
	out = []
	for char in plaintext:
		i = (i + 1) % 256
		j = (j + secret[i]) % 256
		
		swap (secret, j, i)
		
		# Now, we XOR the generated pseudorandom stream byte with the plaintext/ciphertext byte.
		nextIndice = (secret[i] + readAndLeak(secret, j)) % 256
		out.append(chr(ord(char) ^ secret[nextIndice]))
	
	closeIndicesFile()
	open(ciphertext_file_name, 'wb').write(''.join(out))
	
	


if __name__ == "__main__":
	main()
	
